% included from ex_long.pro
constants		% mouse interrupts
/*
follow			=1
nofollow		=0
reset_mouse		=0
enable_cursor		=1
disable_cursor		=2
read_cursor		=3
set_cursor		=4
button_press_state	=5
button_release_state	=6
x_range_of_cursor	=7
y_range_of_cursor	=8
graph_mode		=9
text_mode		=10
read_mouse_motion	=11
*/
PREDICATES
%read_complex_key(INTEGER,KEY)
loop1(KEY,KEY)
mouse_key(KEY)
mouse_key1(INTEGER,KEY)
board_key(KEY)
decision(KEY,KEY,KEY)
not_released(INTEGER,INTEGER)
check_time_and_button(INTEGER,INTEGER)
button_pressed(INTEGER)
not_pressed(INTEGER)
%init_mouse_driver
%del_mouse_cursor
%dsp_mouse_cursor
assert_form(INTEGER)
%set_screen_cursor(ROW,COL)
CLAUSES
/*********************** KEY LOOP ************************/
read_complex_key(_):-dsp_mouse_cursor,fail.
read_complex_key(KEY):-
	!,
	loop1(push,KEY),
	del_mouse_cursor.

loop1(push,RESULT):-!, 		% cut for stack clearning
	mouse_key(MKEY),!,
	board_key(BKEY),
	decision(BKEY,MKEY,STOP),!,
	loop1(STOP,RESULT).
loop1(K,K):-!.

/*************************************************/

mouse_key(KEY):-
	button_pressed(BUTTON),
	BUTTON>0,!,			% pressed BUTTON
	time(_,Minutes,Seconds,H),
	T=(Minutes*60+Seconds)*10+H/10,
	not(not_released(BUTTON,T)),!,
	mouse_key1(BUTTON,KEY),!.
mouse_key(push).

not_released(BUTTON,TIME):-		% fail when released
	repeat,				% wait release button
	bios($33,reg(6,BUTTON,0,0,0,0,0,0),reg(REL,_,_,_,_,_,_,_)),
	bitand(BUTTON,REL,RES),
	check_time_and_button(RES,TIME),!,
%	RES=0,!,
	fail.				% clear heap
check_time_and_button(0,_):-!.
check_time_and_button(_,T):-
	time(_,Minutes,Seconds,H),
	(Minutes*60+Seconds)*10+H/10-T>3,!. % button is pressed longer 3 sec

/************* test button pressed ***************************************/
button_pressed(I):-			% find button pressed
	for(I,0,3),
	not(not_pressed(I)),!.

not_pressed(N):-
	bios($33,reg(3,0,0,0,0,0,0,0),reg(_,BUTTON,_,_,_,_,_,_)),
	BUTTON=(N),!,fail.		% fail if pressed
not_pressed(_).			% true if not pressed

mouse_key1(1,click):-!.
mouse_key1(2,rightclick):-!.
mouse_key1(3,tripple):-!.

/***********************************************/
board_key(KEY):-keypressed,!,readkey(KEY).
board_key(push).	% no keyboard

/***************** decision(KEYBOARD,MOUSE,RESULT)*****************************/
decision(push,push,push):-!.
decision(ctrlbdel,push,push):-retract(mouse_form(_)),!.
decision(ctrlbdel,push,push):-assert(mouse_form(0)),!.
decision(KEY,push,KEY):-!.
decision(push,K,K):-!.

init_mouse_driver:-
		bios($33,reg(0,0,0,0,0,0,0,0),reg(Stat,_,_,_,_,_,_,_)),
		Stat=-1,!,assert_form(0),
		dsp_mouse_cursor;
		true.
del_mouse_cursor:-bios($33,reg(2,0,0,0,0,0,0,0),reg(_,_,_,_,_,_,_,_)),fail;true.
dsp_mouse_cursor:-bios($33,reg(1,0,0,0,0,0,0,0),reg(_,_,_,_,_,_,_,_)),fail;true.

assert_form(F):-retract(mouse_form(_)),fail;
		assert(mouse_form(F)).

PREDICATES
%read_mouse(INTEGER,ROW,COL)
check_pgdn
check_pgup
check_up
check_down
check_menu
check_outside
check_frame(INTEGER,INTEGER)
%move_window
CLAUSES
read_mouse(Stat,ROW,COL):-
		not(mouse_form(_)),	/* mouse absent */
		Stat=0,ROW=1,COL=66,!;
		bios($33,reg(3,0,0,0,0,0,0,0),reg(_,Stat,X1,Y1,_,_,_,_)),
		COL=X1/8,ROW=Y1/8.
check_pgdn:-	read_mouse(_,ROW,COL),
		makewindow(_,FATTR,_,_,R,C,NR,NC),
		check_frame(FATTR,FRAME),	% if frame absent FRAME=-1
		ROW=R+NR-1+FRAME,COL<C+NC,!,fail;% fail if bottom border
		true.
check_pgup:-	read_mouse(_,ROW,COL),
		makewindow(_,_,FATTR,_,R,C,_,NC),
		check_frame(FATTR,FRAME),	% if frame absent FRAME=-1
		ROW=R+FRAME,COL<C+NC,!,fail;	% fail if upper border
		true.
check_up:-	read_mouse(_,ROW,COL),
		makewindow(_,_,_,_,R,C,_,NC),
		ROW=R,COL=C+NC-1,!,fail;	% fail if upper right corner
		true.
check_down:-	read_mouse(_,ROW,COL),
		makewindow(_,_,_,_,R,C,NR,NC),
		ROW=R+NR-1,COL=C+NC-1,!,fail;	% fail if bottom right corner
		true.
check_menu:-	read_mouse(_,ROW,COL),
		makewindow(_,_,_,_,R,C,_,_),
		ROW=R,COL=C,!,fail;		% fail if bottom left corner
		true.
check_outside:-	read_mouse(_,ROW,COL),
		makewindow(_,_,FATTR,_,R,C,NR,NC),
		check_frame(FATTR,FRAME),
		ROW>=R,ROW+1+FRAME<R+NR,
		COL>C,COL<C+NC-1,!,
		RC=ROW-R-1-FRAME,CC=COL-C-1-FRAME,
		max(0,RC,RM),max(0,CC,CM),
		cursor(RM,CM),
		fail;				% fail if inside
		true.

check_frame(0,-1):-!.
check_frame(_,0).

set_screen_cursor(ROW,COL):-
	cursor(ROW,COL),
	mouse_form(_),!;
	makewindow(_,_,FATTR,_,Ymin,Xmin,_,_),FATTR=0,
	Xmouse=(Xmin+COL)*8,Ymouse=(Ymin+ROW)*8,
	bios($33,reg(4,0,Xmouse,Ymouse,0,0,0,0),_),!;
	makewindow(_,_,_,_,Ymin,Xmin,_,_),
	Xmouse=(Xmin+COL+1)*8,Ymouse=(Ymin+ROW+1)*8,
	bios($33,reg(4,0,Xmouse,Ymouse,0,0,0,0),_),!.
